//Replace FullZoom

/**
 * Controls the "full zoom" setting and its site-specific preferences.
 */
FullZoom = {

  //**************************************************************************//
  // Name & Values

  // The name of the setting.  Identifies the setting in the prefs database.
  name: "browser.content.full-zoom",
  mode: "browser.content.full-mode",
  auto: "browser.content.full-AutoFit",
  lastInnerWidth: 0,

  // The global value (if any) for the setting.  Lazily loaded from the service
  // when first requested, then updated by the pref change listener as it changes.
  // If there is no global value, then this should be undefined.

  get globalValue FullZoom_get_globalValue() {
    var globalValue = this._prefBranch.getIntPref("extensions.browser.zoom.fullZoom.default") / 100;
    if (typeof globalValue == "undefined")
      globalValue =  1.0;
    else
      globalValue = this._ensureValid(globalValue);
    delete this.globalValue;
    return this.globalValue = globalValue;
  },
  get globalMode FullZoom_get_globalMode() {
    //this.globalMode ; Text zoom or not
    var globalMode = !this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.Textmode");
    if (typeof globalMode == "undefined")
      globalMode = true;
    delete this.globalMode;
    return this.globalMode = globalMode;
  },
  get globalAuto FullZoom_get_globalAuto() {
    //this.globalMode ; Auto Fit or not
    var globalAuto = this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.fitToWidth");
    if (typeof globalAuto == "undefined")
      globalAuto = false;
    delete this.globalAuto;
    return this.globalAuto = globalAuto;
  },
  get forceFitToWidth FullZoom_get_forceFitToWidth() {
    var forceFitToWidth = this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.forceFitToWidth");
    if (typeof forceFitToWidth == "undefined")
      forceFitToWidth =  true;
    delete this.forceFitToWidth;
    return this.forceFitToWidth = forceFitToWidth;
  },
  //For Bug 416661 Site-specific zoom level shouldn't apply to image documents
  get ignoreImageDocument FullZoom_get_ignoreImageDocument() {
    var ignoreImageDocument = this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.ignoreImageDocument");
    if (typeof ignoreImageDocument == "undefined")
      ignoreImageDocument =  true;
    delete this.ignoreImageDocument;
    return this.ignoreImageDocument = ignoreImageDocument;
  },

  //**************************************************************************//
  // Convenience Getters

  // Content Pref Service
  get _cps FullZoom_get__cps() {
    delete this._cps;
    return this._cps = Cc["@mozilla.org/content-pref/service;1"].
                       getService(Ci.nsIContentPrefService);
  },

  get _prefBranch FullZoom_get__prefBranch() {
    delete this._prefBranch;
    return this._prefBranch = Cc["@mozilla.org/preferences-service;1"].
                              getService(Ci.nsIPrefBranch2);
  },

  // browser.zoom.siteSpecific preference cache
  _siteSpecificPref: undefined,

  // browser.zoom.updateBackgroundTabs preference cache
  updateBackgroundTabs: undefined,

  // whether we are in private browsing mode
  _inPrivateBrowsing: false,

  get siteSpecific FullZoom_get_siteSpecific() {
    return !this._inPrivateBrowsing && this._siteSpecificPref;
  },


  //**************************************************************************//
  // nsISupports

  // We can't use the Ci shortcut here because it isn't defined yet.
  interfaces: [Components.interfaces.nsIDOMEventListener,
               Components.interfaces.nsIObserver,
               Components.interfaces.nsIContentPrefObserver,
               Components.interfaces.nsISupportsWeakReference,
               Components.interfaces.nsISupports],

  QueryInterface: function FullZoom_QueryInterface(aIID) {
    if (!this.interfaces.some(function (v) aIID.equals(v)))
      throw Cr.NS_ERROR_NO_INTERFACE;
    return this;
  },


  //**************************************************************************//
  // Initialization & Destruction

  init: function FullZoom_init() {

    //on load
    // Listen for scrollwheel events so we can save scrollwheel-based changes.
    window.addEventListener("DOMMouseScroll", this, false);

    document.getElementById("cmd_fullZoomReset").
      setAttribute("oncommand","FullZoom.resetZoom()");
    document.getElementById("cmd_fullZoomToggle").
      setAttribute("oncommand","FullZoom.toggleZoom()");
    fullZoomBtn.addPrefListener(fullZoomBtn.PrefListener);

    // Register ourselves with the service so we know when our pref changes.
    this._cps.addObserver(this.name, this);
    this._cps.addObserver(this.mode, this);

    // We disable site-specific preferences in Private Browsing mode, because the
    // content preferences module is disabled
    let os = Cc["@mozilla.org/observer-service;1"].
             getService(Ci.nsIObserverService);
    os.addObserver(this, "private-browsing", true);

    try {
      // Retrieve the initial status of the Private Browsing mode.
      this._inPrivateBrowsing = Cc["@mozilla.org/privatebrowsing;1"].
                                getService(Ci.nsIPrivateBrowsingService).
                                privateBrowsingEnabled;
    } catch (e) {
      this._inPrivateBrowsing = false;
    }

    this._siteSpecificPref =
      this._prefBranch.getBoolPref("browser.zoom.siteSpecific");
    this.localFolderSpecific =
      this._prefBranch.getBoolPref("extensions.browser.zoom.localFolderSpecific");
    try {
      this.updateBackgroundTabs =
        this._prefBranch.getBoolPref("browser.zoom.updateBackgroundTabs");
    } catch (e) { }
    // Listen for changes to the browser.zoom branch so we can enable/disable
    // updating background tabs and per-site saving and restoring of zoom levels.
    this._prefBranch.addObserver("browser.zoom.", this, true);
    this._prefBranch.addObserver("extensions.browser.zoom.", this, true);

    //When the default browser confirmation dialog is showwn on startup, Site specific zoom does not be allpied.
    //So, Force apply for the first tab
    if (gBrowser.mTabs.length == 1) {
      this.onLocationChange(gBrowser.currentURI, true, gBrowser.selectedBrowser);
    }
  },

  destroy: function FullZoom_destroy() {
    let os = Cc["@mozilla.org/observer-service;1"].
             getService(Ci.nsIObserverService);
    try { os.removeObserver(this, "private-browsing"); } catch(e) { }
    this._prefBranch.removeObserver("browser.zoom.", this);
    this._prefBranch.removeObserver("extensions.browser.zoom.", this);
    this._cps.removeObserver(this.name, this);
    this._cps.removeObserver(this.mode, this);
    window.removeEventListener("DOMMouseScroll", this, false);

    fullZoomBtn.removePrefListener(fullZoomBtn.PrefListener);
    delete this._cps;
  },


  //**************************************************************************//
  // Event Handlers

  // nsIDOMEventListener

  handleEvent: function FullZoom_handleEvent(event) {
    switch (event.type) {
      case "DOMMouseScroll":
        this._handleMouseScrolled(event);
        break;
    }
  },

  _handleMouseScrolled: function FullZoom__handleMouseScrolled(event) {
    // Construct the "mousewheel action" pref key corresponding to this event.
    // Based on nsEventStateManager::GetBasePrefKeyForMouseWheel.
    var pref = "mousewheel";
    if (typeof MOUSE_SCROLL_IS_HORIZONTAL != 'undefined') { //3.0.*
      if (event.scrollFlags & MOUSE_SCROLL_IS_HORIZONTAL)
        pref += ".horizscroll";
    } else { //Fx3.1.*
      if (event.axis == event.HORIZONTAL_AXIS)
        pref += ".horizscroll";
    }

    if (event.shiftKey)
      pref += ".withshiftkey";
    else if (event.ctrlKey)
      pref += ".withcontrolkey";
    else if (event.altKey)
      pref += ".withaltkey";
    else if (event.metaKey)
      pref += ".withmetakey";
    else
      pref += ".withnokey";

    pref += ".action";

    // Don't do anything if this isn't a "zoom" scroll event.
    var isZoomEvent = false;
    try {
      isZoomEvent = (gPrefService.getIntPref(pref) == MOUSE_SCROLL_ZOOM);
    } catch (e) {}
    if (!isZoomEvent)
      return;

    // XXX Lazily cache all the possible action prefs so we don't have to get
    // them anew from the pref service for every scroll event?  We'd have to
    // make sure to observe them so we can update the cache when they change.

    // We have to call _applySettingToPref in a timeout because we handle
    // the event before the event state manager has a chance to apply the zoom
    // during nsEventStateManager::PostHandleEvent.
    window.setTimeout(function (self) { self._applySettingToPref() }, 0, this);
  },

  // nsIObserver

  observe: function (aSubject, aTopic, aData) {
    switch (aTopic) {
      case "nsPref:changed":
        switch (aData) {
          case "browser.zoom.siteSpecific":
            this._siteSpecificPref =
              this._prefBranch.getBoolPref("browser.zoom.siteSpecific");
            break;
          case "extensions.browser.zoom.localFolderSpecific":
            this.localFolderSpecific =
              this._prefBranch.getBoolPref("extensions.browser.zoom.localFolderSpecific");
            break;
          case "browser.zoom.updateBackgroundTabs":
            this.updateBackgroundTabs =
              this._prefBranch.getBoolPref("browser.zoom.updateBackgroundTabs");
            break;
          case "extensions.browser.zoom.fullZoom.default":
            this.globalValue =
              this._prefBranch.getIntPref("extensions.browser.zoom.fullZoom.default") / 100;
              this._cps.setPref(null, this.name, this.globalValue);
            break;
          case "extensions.browser.zoom.fullZoom.Textmode":
            this.globalMode =
              !this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.Textmode");
            break;
          case "extensions.browser.zoom.fullZoom.fitToWidth":
            this.globalAuto =
              this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.fitToWidth");
            break;
          case "extensions.browser.zoom.fullZoom.forceFitToWidth":
            this.forceFitToWidth =
              this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.forceFitToWidth");
            break;
          case "extensions.browser.zoom.fullZoom.ignoreImageDocument":
            this.ignoreImageDocument =
              this._prefBranch.getBoolPref("extensions.browser.zoom.fullZoom.ignoreImageDocument");
            break;
        }
        break;
      case "private-browsing":
        switch (aData) {
          case "enter":
            this._inPrivateBrowsing = true;
            break;
          case "exit":
            this._inPrivateBrowsing = false;
            break;
        }
        break;
    }
  },

  //Ensure local file url convert to aURI
  convURI: function(aURI) {
    const ioService = Components.classes['@mozilla.org/network/io-service;1']
                      .getService(Components.interfaces.nsIIOService);
    if (!!aURI && !this.localFolderSpecific) {
      if (/^file:/i.test(aURI.spec)) {
        var tmp = aURI.spec.split('/');
        tmp.pop();
        var url = tmp.join('/');
        aURI = ioService.newURI(url, null, null)
      }
    }
    return aURI;
  },

  // nsIContentPrefObserver
  onContentPrefSet: function FullZoom_onContentPrefSet(aGroup, aName, aValue) {
    var url = this.convURI(gBrowser.currentURI);
    if (aGroup == this._cps.grouper.group(url))
      this._applyPrefToSetting(aName, aValue);
    else if (aGroup == null) {
      this.globalValue = this._ensureValid(aValue);

      // If the current page doesn't have a site-specific preference,
      // then its zoom should be set to the new global preference now that
      // the global preference has changed.
      if (!this._cps.hasPref(url, aName))
        this._applyPrefToSetting(aName);
    }
  },

  onContentPrefRemoved: function FullZoom_onContentPrefRemoved(aGroup, aName) {
    var url = this.convURI(gBrowser.currentURI);
    if (aGroup == this._cps.grouper.group(url))
      this._applyPrefToSetting(aName);
    else if (aGroup == null) {
      this.globalValue = undefined;

      // If the current page doesn't have a site-specific preference,
      // then its zoom should be set to the default preference now that
      // the global preference has changed.
      if (!this._cps.hasPref(url, aName))
        this._applyPrefToSetting(aName);
    }
  },

  /**
   * Called when the location of a tab changes.
   * When that happens, we need to update the current zoom level if appropriate.
   *
   * @param aURI
   *        A URI object representing the new location.
   * @param aIsTabSwitch
   *        Whether this location change has happened because of a tab switch.
   * @param aBrowser
   *        (optional) browser object displaying the document
   */
  onLocationChange: function FullZoom_onLocationChange(aURI, aIsTabSwitch, aBrowser) {
    if (!aURI || (aIsTabSwitch && !this.siteSpecific))
      return;
    if (gInPrintPreviewMode)
      return;
    /*
    if (aURI.spec == 'about:blank')
      return;
    */
    var browser = aBrowser || gBrowser.selectedBrowser;
    aURI = this.convURI(aURI);

    if (this.ignoreImageDocument && browser.contentDocument instanceof Ci.nsIImageDocument) {
      ZoomManager.setZoomForBrowser(browser, 1.0);
    } else {
      if (!this.siteSpecific) {
        if (this.isAlreadyApplied(browser)) {
          //Get saved zoomlevel for Tab and applied, if siteSpecific is false.
          var useFullAuto = this.getApplied(browser);
          var index = gBrowser.getBrowserIndexForDocument(browser.contentDocument);
          var aTab = gBrowser.mTabs[index];
          var SavedURL = aTab.getAttribute("FullZoomAutoSavedURL");
//fullZoomBtn.debug("SavedURL " + SavedURL + "\nlChange URI.spec " + aURI.spec);
          if ( this.globalAuto && useFullAuto && SavedURL != aURI.spec) {
            if (browser == gBrowser.selectedBrowser)
              fullZoomBtn.doFullZoomBy(-1, true);
          }

        } else {
          //set default zoomlevel for Tab, if siteSpecific is false.
          var useFullZoom = this.globalMode;
          ZoomManager.useFullZoom = useFullZoom;
          ZoomManager.setZoomForBrowser(browser, this.globalValue);
          ZoomManager.useFullZoom = useFullZoom;
          ZoomManager.useFullAuto = false;
          //Save zoomlevel for Tab, if siteSpecific is false.
          this.setApplied(browser);
        }
      } else {
          this._applyPrefToSetting(this.mode, this._cps.getPref(aURI, this.mode), browser);
          this._applyPrefToSetting(this.name, this._cps.getPref(aURI, this.name), browser);
          this._applyPrefToSetting(this.auto, this._cps.getPref(aURI, this.auto), browser);

          if ( this.globalAuto && ZoomManager.useFullAuto)
            if (browser == gBrowser.selectedBrowser &&
                (typeof gBrowser.selectedBrowser.FullZoomAutoSavedURL == 'undefined' ||
                 gBrowser.selectedBrowser.FullZoomAutoSavedURL != aURI.spec)) {
              fullZoomBtn.doFullZoomBy(-1, true);
              gBrowser.selectedBrowser.FullZoomAutoSavedURL = aURI.spec;
            }
      }
    }
    if (browser == gBrowser.selectedBrowser) {
      fullZoomBtn.showZoomLevelInStatusbar();
//alert("onLocationChange" + (ZoomManager.useFullZoom?"F":"T") + ZoomManager.zoom*100);
      }
  },

  //When not Site Specific Mode, Does already applied zooming facor for aBrowser or not.
  isAlreadyApplied: function FullZoom_isAlreadyApplied(aBrowser) {
    var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].
                               getService(Components.interfaces.nsISessionStore);

    var browser = aBrowser || gBrowser.selectedBrowser;
    var index = gBrowser.getBrowserIndexForDocument(browser.contentDocument);
    var aTab = gBrowser.mTabs[index];

    try {
      if (!!ss.getTabValue(aTab, "FullZoomMode")) {
        return true;
      }
    } catch(ex) {
     return aTab.hasAttribute("FullZoomMode");
    }
  },

  //When not Site Specific Mode, Get ZoomLevel for curennt aBrowser
  getApplied: function FullZoom_getApplied(aBrowser) {
    var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].
                               getService(Components.interfaces.nsISessionStore);

    var browser = aBrowser || gBrowser.selectedBrowser;
    var index = gBrowser.getBrowserIndexForDocument(browser.contentDocument);
    var aTab = gBrowser.mTabs[index];

    var value, useFullZoom, useFullAuto;
    try {
      value       = ss.getTabValue(aTab, "FullZoomLevel");
      useFullZoom = ss.getTabValue(aTab, "FullZoomMode") == 'true';
      useFullAuto = ss.getTabValue(aTab, "FullZoomAuto") == 'true';
    } catch(ex) {
      value       = aTab.getAttribute("FullZoomLevel");
      useFullZoom = aTab.getAttribute("FullZoomMode") == 'true';
      useFullAuto = aTab.getAttribute("FullZoomAuto") == 'true';
    }
    ZoomManager.useFullZoom = useFullZoom;
    if (value)
      ZoomManager.setZoomForBrowser(browser, this._ensureValid(value));
    else
      ZoomManager.setZoomForBrowser(browser, this.globalValue);
    ZoomManager.useFullZoom = useFullZoom;
    ZoomManager.useFullAuto = useFullAuto;
    return useFullAuto;
  },

  //When not Site Specific Mode, Save ZoomLevel for curennt aBrowser
  setApplied: function FullZoom_setApplied(aBrowser) {
    var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].
                               getService(Components.interfaces.nsISessionStore);

    var browser = aBrowser || gBrowser.selectedBrowser;
    var index = gBrowser.getBrowserIndexForDocument(browser.contentDocument);
    var aTab = gBrowser.mTabs[index];

    try {
      ss.setTabValue(aTab, "FullZoomLevel", ZoomManager.getZoomForBrowser(browser));
      ss.setTabValue(aTab, "FullZoomMode", !!ZoomManager.useFullZoom);
      ss.setTabValue(aTab, "FullZoomAuto", !!ZoomManager.useFullAuto);
    } catch(ex) {}
    aTab.setAttribute("FullZoomLevel", ZoomManager.getZoomForBrowser(browser));
    aTab.setAttribute("FullZoomMode", !!ZoomManager.useFullZoom);
    aTab.setAttribute("FullZoomAuto", !!ZoomManager.useFullAuto);
    aTab.setAttribute("FullZoomAutoSavedURL", browser.currentURI.spec);
  },

  //When not Site Specific Mode, Remove ZoomLevel for curennt aBrowser
  removeApplied: function FullZoom_removeApplied() {
    var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].
                               getService(Components.interfaces.nsISessionStore);
    var aTab = gBrowser.mCurrentTab;
    try {
      ss.deleteTabValue(aTab, "FullZoomLevel");
      ss.deleteTabValue(aTab, "FullZoomMode");
      ss.deleteTabValue(aTab, "FullZoomAuto");
      ss.deleteTabValue(aTab, "FullZoomAutoSavedURL");
    } catch(ex) {}
    aTab.removeAttribute("FullZoomLevel");
    aTab.removeAttribute("FullZoomMode");
    aTab.removeAttribute("FullZoomAuto");
    aTab.removeAttribute("FullZoomAutoSavedURL");
  },

  // update state of zoom type menu item
  updateMenu: function FullZoom_updateMenu() {
    var menuItem = document.getElementById("toggle_zoom");

    menuItem.setAttribute("checked", !ZoomManager.useFullZoom);
  },

  //**************************************************************************//
  // Setting & Pref Manipulation

  reduce: function FullZoom_reduce() {
    ZoomManager.reduce();
    this._applySettingToPref();
  },

  enlarge: function FullZoom_enlarge() {
    ZoomManager.enlarge();
    this._applySettingToPref();
  },

  toggleZoom: function FullZoom_enlarge() {
    ZoomManager.toggleZoom();
    this._applySettingToPref();
  },

  resetZoom: function FullZoom_enlarge() {
    ZoomManager.reset();
    this._applySettingToPref();
  },

  reset: function FullZoom_reset() {
    if (typeof this.globalValue != "undefined")
      ZoomManager.zoom = this.globalValue;
    else
      ZoomManager.reset();

    if (!(this.ignoreImageDocument && content.document instanceof Ci.nsIImageDocument)) {
      this._removePref();

      //Remove saved zoomlevel for Tab
      this.removeApplied();
    }
  },

  setSettingValue: function FullZoom_setSettingValue(aBrowser) {
    var browser = aBrowser || gBrowser.selectedBrowser;
    var uri = this.convURI(browser.currentURI);

    var value = this._cps.getPref(uri, this.name);
    this._applyPrefToSetting(this.name ,value, browser);

    value = this._cps.getPref(uri, this.mode);
    this._applyPrefToSetting(this.mode ,value, browser);
  },


  /**
   * Set the zoom level for the current tab.
   *
   * Per nsPresContext::setFullZoom, we can set the zoom to its current value
   * without significant impact on performance, as the setting is only applied
   * if it differs from the current setting.  In fact getting the zoom and then
   * checking ourselves if it differs costs more.
   *
   * And perhaps we should always set the zoom even if it was more expensive,
   * since DocumentViewerImpl::SetTextZoom claims that child documents can have
   * a different text zoom (although it would be unusual), and it implies that
   * those child text zooms should get updated when the parent zoom gets set,
   * and perhaps the same is true for full zoom
   * (although DocumentViewerImpl::SetFullZoom doesn't mention it).
   *
   * So when we apply new zoom values to the browser, we simply set the zoom.
   * We don't check first to see if the new value is the same as the current
   * one.
   **/
  _applyPrefToSetting: function FullZoom__applyPrefToSetting(aName, aValue, aBrowser) {
    var browser = aBrowser || gBrowser.selectedBrowser;

    var resetZoom = (!this.siteSpecific || gInPrintPreviewMode ||
                     browser.contentDocument instanceof Ci.nsIImageDocument);

    if (aName == this.name) {
      try {
        if (resetZoom)
          ZoomManager.setZoomForBrowser(browser, this.globalValue);
        else if (typeof aValue != "undefined")
          ZoomManager.setZoomForBrowser(browser, this._ensureValid(aValue));
        else
          ZoomManager.setZoomForBrowser(browser, this.globalValue);
      }
      catch(ex) {}
    } else if (aName == this.mode) {
      try {
        if (resetZoom)
          ZoomManager.useFullZoom = this.globalMode;
        else if (typeof aValue != "undefined")
          ZoomManager.useFullZoom = !!aValue;
        else
          ZoomManager.useFullZoom = this.globalMode;
      }
      catch(ex) {}
    } else if (aName == this.auto) {
      try {
        if (resetZoom)
          ZoomManager.useFullAuto = false;
        else if (typeof aValue != "undefined")
          ZoomManager.useFullAuto = !!aValue;
        else
          ZoomManager.useFullAuto = false;
      }
      catch(ex) {}
    }
    if (browser == gBrowser.selectedBrowser)
      fullZoomBtn.showZoomLevelInStatusbar();
  },

  _applySettingToPref: function FullZoom__applySettingToPref(aBrowser) {
    if (gInPrintPreviewMode)
      return;

    var browser = aBrowser || gBrowser.selectedBrowser;
    var url = this.convURI(browser.currentURI);

    if (!(this.ignoreImageDocument && browser.contentDocument instanceof Ci.nsIImageDocument)) {
      if (!this.siteSpecific) {
        //Save zoomlevel for Tab
        this.setApplied(browser);
      } else {
        if (Math.floor(ZoomManager.getZoomForBrowser(browser) * 100) != Math.floor(this.globalValue * 100) ) {
          this._cps.setPref(url, this.name, ZoomManager.getZoomForBrowser(browser));
        } else {
          this._removePrefZoom(url);
        }
        if (ZoomManager.useFullZoom != this.globalMode ) {
          this._cps.setPref(url, this.mode, ZoomManager.useFullZoom);
        } else {
          this._removePrefMode(url);
        }
        if (!!ZoomManager.useFullAuto) {
          this._cps.setPref(url, this.auto, ZoomManager.useFullAuto);
        } else {
          this._removePrefAuto(url);
        }
      }
    }

    if (browser == gBrowser.selectedBrowser)
      fullZoomBtn.showZoomLevelInStatusbar();
  },

  _removePref: function FullZoom_removePref(url) {
    if (!url)
      url = this.convURI(gBrowser.currentURI);
    this._cps.removePref(url, this.mode);
    this._cps.removePref(url, this.name);
    this._cps.removePref(url, this.auto);
  },

  _removePrefZoom: function FullZoom_removePrefZoom(url) {
    if (!url)
      url = this.convURI(gBrowser.currentURI);
    this._cps.removePref(url, this.name);
  },

  _removePrefMode: function FullZoom_removePrefMode(url) {
    if (!url)
      url = this.convURI(gBrowser.currentURI);
    this._cps.removePref(url, this.mode);
  },

  _removePrefAuto: function FullZoom_removePrefAuto(url) {
    if (!url)
      url = this.convURI(gBrowser.currentURI);
    this._cps.removePref(url, this.auto);
  },


  //**************************************************************************//
  // Utilities

  _ensureValid: function FullZoom__ensureValid(aValue) {
    if (isNaN(aValue))
      return this.globalValue;

    if (aValue < ZoomManager.MIN)
      return ZoomManager.MIN;

    if (aValue > ZoomManager.MAX)
      return ZoomManager.MAX;

    return aValue;
  }
};
